///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2010 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Jan 06, 2010	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import com.tietoenator.trip.jxp.*;
import com.tietoenator.trip.jxp.data.*;
import com.tietoenator.trip.jxp.database.TdbDatabaseDesign;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbLocalSession;
import com.tietoenator.trip.jxp.session.TdbSession;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;

/**
 * This class demonstrates how to use the TRIPxml functionality to import
 * files and documents into a TRIPxml database.
 */
public class XmlExport
{
    private TdbSession m_session;
    private TdbXmlRecord m_rec;
    private TdbDatabaseDesign m_design;

    /**
     * Constructor
     * @throws TdbException
     */
    public XmlExport()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** TRIPxml Import Example ****", false);
    }


    public void close()
    {
        if (m_rec != null)
        {
            // Calling clear is VERY important if HTTP stream download has been
            // used! If we forget, we will end up with a bunch of hanging threads.

            m_rec.clear(true);
            m_rec = null;
        }
        m_design = null;

        try
        {
            if (null != m_session)
                m_session.logout();
        }
        catch (Exception ignore) {}
        m_session = null;
    }

   boolean arrayContains(String str, String[] arr)
   {
       for (String item : arr)
       {
           if (item.equalsIgnoreCase(str))
               return true;
       }
       return false;
   }


    TdbDatabaseDesign selectDatabase(Console stdin) throws TdbException
    {
        String db;
        TdbDatabaseDesign design = null;

        do
        {
            db = null;
            while (db == null)
                db = stdin.readln("XML Database", null);

            design = new TdbDatabaseDesign(m_session);
            design.get(db);

            // NOTE: Versions older than 6.2-0 does not return the xml indicator
            // unless we are logged on as the database owner.

            if (!design.isXmlEnabled())
            {
                System.err.println("\nERROR: The database "+design.getName().toUpperCase()+" is not XML enabled.\n");
                design = null;
            }
        }
        while (design == null);
        return design;
    }

    String selectImportFile(Console stdin) throws TdbException
    {
        File f = null;
        String fname = null;
        do
        {
            while (fname == null)
                fname = stdin.readln("File to export to",null);

            f = new File(fname);
            if (f.isDirectory())
            {
                System.err.println("\nERROR: specified file does not exist or is not a file.\n");
                f = null;
            }
        }
        while (f == null);
        return fname;
    }

    void process() throws TdbException, FileNotFoundException
    {
        Console stdin = new Console();
        String strQuery = null;
        boolean bHighlight = false;
        int nRid = 0;

        // Select an XML database.
        m_design = selectDatabase(stdin);

        // Speciy how to look for a record
        if (0 != stdin.readint("Search for record (1) or specify rid (0)",0,true))
        {
            while (strQuery == null)
                strQuery = stdin.readln("FIND order (include FIND or FUZZ)", null);

            // Highlight the hits?
            if (0 != stdin.readint("Apply hit markup? ( Yes=1 No=0 )", 0, true))
                bHighlight = true;
        }
        else
        {
            while (nRid<1)
                nRid = stdin.readint("Record ID", 0, true);
        }

        String strExportFile = null;
        while (strExportFile == null)
            strExportFile = stdin.readln("File to export to",null);

        OutputStream stream = null;
        File exportFile = new File(strExportFile);

        if (!exportFile.isDirectory())
        {
            // Are we going to export to a Java stream?
            if (0 != stdin.readint("OutputStream export ( Yes=1 No=0 )", 0, true))
                stream = new FileOutputStream(exportFile);
        }


        // Determine what I/O mode is valid for the current setup.

        String[] validModes;
        if ((m_session instanceof TdbLocalSession))
            validModes = new String[]{"inline","stream","file"};
        else
            validModes = new String[]{"inline","stream"};

        // Build the prompt string for asking about what I/O mode to select

        StringBuilder strModePrompt = new StringBuilder("I/O Mode (");
        for (int m=0;m<validModes.length;m++)
        {
            if (m>0)
                strModePrompt.append(",");
            strModePrompt.append(validModes[m]);
        }
        strModePrompt.append(")");

        // Select I/O mode

        String strIOMode = null;
        TdbXmlRecord.IOMode ioMode = TdbXmlRecord.IOMode.Inline;
        do
        {
            while (strIOMode == null)
                strIOMode = stdin.readln(strModePrompt.toString(), "inline");
            if (!arrayContains(strIOMode,validModes))
                strIOMode = null;
        }
        while (strIOMode == null);

        if (strIOMode.equalsIgnoreCase("stream"))
            ioMode = TdbXmlRecord.IOMode.Stream;
        else if (strIOMode.equalsIgnoreCase("file"))
            ioMode = TdbXmlRecord.IOMode.File;
        else
            ioMode = TdbXmlRecord.IOMode.Inline;


        // Create a TdbXmlRecord object (subclass of TdbRecord)
        if (nRid>0)
        {
            m_rec = new TdbXmlRecord(m_session,m_design,false);

            // The ID of the record to export.
            m_rec.setRecordId(nRid);

            // NOTE: If your TRIPxml database has custom fields, you may at
            // this point update those custom fields, just like you would
            // do with a regular TdbRecord object.

            // Override various TRIPxml export options.
            // Do this BEFORE the call to PrepareImport!
            m_rec.setInputOutputMode(ioMode);

            // Prepare the TdbXmlRecord instance to perform a TRIPxml export
            // upon the next call to get().
            if (stream != null)
                m_rec.prepareExport(stream);
            else
                m_rec.prepareExport(strExportFile);

            // Now perform the actual export.
            m_rec.get();
        }
        else
        {
            // Perform the specified search
            TdbRecordSet rs = new TdbRecordSet(m_session);
            rs.setFrom(1);
            rs.setTo(1);
            rs.setDatabase(m_design.getName());
            rs.setQuery(strQuery);

            // Create retrieval template for TRIPxml export.
            m_rec = new TdbXmlRecord(m_session,m_design,false);
            m_rec.setHitMarkup(bHighlight);
            m_rec.setInputOutputMode(ioMode);
            if (stream != null)
                m_rec.prepareExport(stream);
            else
                m_rec.prepareExport(strExportFile);
            rs.setRetrievalTemplate(m_rec);

            // Perform the search. Upon successful completion (and we actually
            // found something), the record has been exported to the specified
            // file.
            rs.get();
        }

        System.out.println("Document exported");
    }

    /**
     * Main entry point
     * @param args Not used
     */
    public static void main(String[] args)
    {
        XmlExport app = null;
        try
        {
            app = new XmlExport();
            app.process();
        }
        catch (TdbException e)
        {
            System.out.println(e.getDebugMessage());
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        finally
        {
            if (null != app)
                app.close();
        }
    }
}
