///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2010 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Jul 20, 2010	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.database;

import com.tietoenator.trip.jxp.TdbControlObject;
import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.control.TdbDatabaseList;
import com.tietoenator.trip.jxp.database.TdbDatabaseClusterDesign;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * Simple example showing how to manage a cluster. This example program includes
 * listing of clusters, listing of cluster members, creation of clusters,
 * deletion of clusters, adding of members to clusters and removal of members
 * from clusters.
 */
public class ClusterAdm
{
   // Show any success message that might be forthcoming from the server

   private static void showMessages(TdbMessageProvider prv)
   {
      for (String msg : prv.messageList())
      {
         System.out.println(msg);
      }
      prv.resetMessages();
   }

   /**
    * Retrieve a list of databases, thesauri and clusters from the server
    * and print the clusters to stdout.
    *
    * @param session Active TdbSession object
    */
   private static void listClusters(TdbSession session) throws TdbException
   {
       TdbDatabaseList dbl = new TdbDatabaseList(session);
       for (TdbControlObject o : dbl)
       {
           if (o.isCluster())
               System.out.println(o.getName());
       }
   }

   /**
    * Main entry point
    * @param args Not used
    */
   public static void main(String[] args)
   {
      try
      {
         TdbSession session = SessionFactory.newInstance("**** Cluster Administration ****", false);
         TdbDatabaseClusterDesign cluster = new TdbDatabaseClusterDesign(session);
         Console stdin = new Console();
         String op, cname="", mname="";
         char cOp;


         // Prompt for mode of operation
         do
         {
             do
             {
                 op = stdin.readln("L(ist),S(how),C(reate),D(elete),A(dd),R(emove)","");
             }
             while (op.length() == 0);

             cOp = Character.toUpperCase(op.charAt(0));
             if (cOp!='L' && cOp!='C' && cOp!='D' && cOp!='R' && cOp!='A' && cOp!='S')
             {
                 System.out.println();
                 System.out.println("ERROR: Invalid operation!");
                 System.out.println();
                 op = "";
             }
         }
         while (op.length() == 0);

         if (cOp != 'L')
         {
             do
             {
                 // Prompt for cluster name
                 do
                 {
                    cname = stdin.readln("Cluster Name", "");
                 }
                 while (cname==null || cname.length()<1);

                 // Unless we're creating a new cluster, we need to fetch the
                 // cluster design from the server. Do that now.
                 if (cOp != 'C')
                 {
                     try
                     {
                         cluster.get(cname);
                     }
                     catch (TdbException tdbx)
                     {
                         System.out.println();
                         System.out.println("ERROR: Could not get cluster design from server!");
                         System.out.println(tdbx.getMessage());
                         System.out.println();
                         cname = "";
                     }
                 }
             }
             while (cname.length()<1);
         }

         if (cOp=='A' || cOp=='R')
         {
             // Prompt for name of cluster member to add or remove.
             do
             {
                mname = stdin.readln("Member Name", "");
             }
             while (mname.length()<1);
         }

         System.out.println();

         if (cOp == 'L')
         {
             // List all clusters
             listClusters(session);
         }
         else if (cOp == 'C')
         {
             // Create a new (empty) cluster
             cluster.put(cname);
         }
         else if (cOp == 'S')
         {
             // Display cluster members and indicate the type of each member.
             System.out.println("Cluster members:");
             for (TdbControlObject o : cluster.members())
             {
                 char cType = 'D';
                 if (o.isThesaurus())
                     cType = 'T';
                 else if (o.isCluster())
                     cType = 'C';
                 System.out.println("   ["+cType+"] "+o.getName());
             }
         }
         else if (cOp == 'D')
         {
             // Delete the cluster
             cluster.delete();
         }
         else if (cOp == 'A')
         {
             // Add a new member to the cluster.
             cluster.add(mname);
             cluster.put();
         }
         else if (cOp == 'R')
         {
             // Remove a member from the cluster.
             cluster.remove(mname);
             cluster.put();
         }

         // Display any messages from the server.
         if (cluster != null)
             showMessages(cluster);

         // Terminate the session
         session.logout();
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }
}
