///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2014 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Mar 25, 2014	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.graph;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.data.TdbGraphRecord;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.graph.TdbGraph;
import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * Shows how to add a vertex to a TRIP graph database. Note that any record
 * can act as a vertex in a TRIP graph. Creating a vertex record explicitly 
 * this way is optional.
 * 
 * THIS CLASS USES EXPERIMENTAL FEATURES. USE IT WITH CAUTION!
 */
public class CreateVertex 
{
    private TdbSession m_session;
    
    public CreateVertex()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** CreateVertex TRIP graph example ****", false);
    }
    
    public void close()
    {
        if (m_session != null)
        {   
            try
            {
                m_session.logout();
            }
            catch (TdbException ignore)
            {
            }
            finally
            {
                m_session = null;
            }
        }
    }


   // Show any success message that might be forthcoming from the server
    private static void showMessages(TdbMessageProvider prv) 
    {
        for (String msg : prv.messageList()) 
        {
            System.out.println(msg);
        }
        prv.resetMessages();
    }

    private String readnoblank(Console stdin, String prefix, String defaultValue)
            throws TdbException 
    {
        String inval;
        do 
        {
            inval = stdin.readln(prefix, defaultValue, true);
        } 
        while (inval.length() == 0);
        return inval;
    }

    public void process(Console stdin) throws TdbException 
    {
        TdbGraph graph;
        TdbRecord refrec;
        String graphDbName;
        String refDbName;
        String vertexLabel;
        int refRid;

        // The graph database to use
        graphDbName = readnoblank(stdin, "Graph database", "GRAPHBASE");
        graph = new TdbGraph(m_session, graphDbName);

        // The name of the edge
        vertexLabel = readnoblank(stdin, "Vertex label", "");

        if (stdin.readbool("Reference an existing record?", false, true)) 
        {
            refDbName = readnoblank(stdin, "Database for referenced record", "");
            refrec = new TdbRecord(m_session, refDbName, false);

            do 
            {
                refRid = stdin.readint("Referenced record ID", 0, true);
            } 
            while (refRid < 1);

            refrec.setRecordId(refRid);

            // Not required but useful for verify if record exists.
            refrec.get();
        } 
        else 
        {
            refrec = null;
        }
        TdbGraphRecord gr = graph.createVertex(vertexLabel, refrec, true);

        showMessages(graph);
        System.out.println();
        System.out.println("Created graph vertex \"" + gr.getName() + "\".");
        System.out.println("     Record ID = " + gr.getRecordId());
        System.out.println("   Record Name = " + gr.getRecordName());

        graph.close();
    }

   
   public static void main(String[] args)
   {
       CreateVertex app = null;
       Console stdin = new Console();
       try
       {
           app = new CreateVertex();
           app.process(stdin);
       }
       catch (TdbException tx)
       {
           System.err.println(tx.getDebugMessage());
       }
       catch (Exception ox)
       {
           ox.printStackTrace(System.err);
       }
       finally
       {
           if (app != null)
               app.close();
       }
   }
}
