///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2013 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Feb 20, 2013	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.graph;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.data.TdbRecord;
import com.tietoenator.trip.jxp.data.TdbSearch;
import com.tietoenator.trip.jxp.data.TdbSearchSet;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;

import com.tietoenator.trip.jxp.graph.TdbGraph;

import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * Basic example of how to use the graph classes in TRIPjxp.
 * 
 * THIS CLASS USES EXPERIMENTAL FEATURES. USE IT WITH CAUTION!
 */
public class Simple
{
    private TdbSession m_session;
    
    public Simple()
            throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** Simple TRIP graph example ****", false);
    }
    
    public void close()
    {
        if (m_session != null)
        {
            
            try
            {
                m_session.logout();
            }
            catch (TdbException ignore)
            {
            }
            finally
            {
                m_session = null;
            }
        }
    }


   // Show any success message that might be forthcoming from the server

   private static void showMessages(TdbMessageProvider prv)
   {
      for (String msg : prv.messageList())
      {
         System.out.println(msg);
      }
      prv.resetMessages();
   }
   

    private String readnoblank(Console stdin, String prefix, String defaultValue)
           throws TdbException
   {
       String inval;
       do
       {
           inval = stdin.readln(prefix,defaultValue,true);
       }
       while (inval.length()==0);
       return inval;
       
   }
   
   
   public void process(Console stdin) throws TdbException
   {
       TdbGraph graph;
       TdbSearchSet resultSet;
       String graphDbName;
       String sourceDbName;
       String edgeName;
       int sourceRid;
       int sourceSet;
       
       TdbSearch normalSearch = new TdbSearch(m_session);
       normalSearch.execute("DEFINE FIND MIN=0");

       // The graph database to use
       graphDbName = readnoblank(stdin,"Graph database","GRAPHBASE");
       graph = new TdbGraph(m_session, graphDbName);
       
       // Database for source record to explore relations from
       sourceDbName = readnoblank(stdin,"Database for source record","ALICE");       
       normalSearch.execute("BASE "+sourceDbName);
       
       // ID of source record to explore relations from
       sourceRid = stdin.readint("Source record ID",1,true);
       normalSearch.execute("FIND R="+sourceRid);
       sourceSet = normalSearch.getLastSearchSet().getSearchId();

       // The name of the relation
       edgeName = readnoblank(stdin,"Edge name (relation)","FRIEND");

       // Execute our graph query
       resultSet = graph.getGraphSet()
                        .source(sourceSet)
                        .follow(edgeName, 0)
                        .resolveTargets();
       
       showMessages(graph);
       
       System.out.println();
       System.out.println(Integer.toString(resultSet.getRecordCount())+" records found.\n");
       for (TdbRecord r : resultSet)
       {
           System.out.println(r.getDatabaseName()+"."+Integer.toString(r.getRecordId()));
       }
       
       normalSearch.close();
       graph.close();
   }
   
   
   public static void main(String[] args)
   {
       Simple app = null;
       Console stdin = new Console();
       try
       {
           app = new Simple();
           app.process(stdin);
       }
       catch (TdbException tx)
       {
           System.err.println(tx.getDebugMessage());
       }
       catch (Exception ox)
       {
           ox.printStackTrace(System.err);
       }
       finally
       {
           if (app != null)
               app.close();
       }
   }
}
